# Packages required - install as needed:
# dplyr, readr, data.table, dtplyr


#### Base R ####
# Sample base R code to merge 2 data frames on a single common column
# Include all rows from one data frame and only matching rows from the other

# Data downloaded from http://bit.ly/USFlightDelays . See InfoWorld article at https://www.infoworld.com/article/3454356/how-to-merge-data-in-r-3-ways-base-r-dplyr-and-datatable.html for more details.

# Code assumes the data resides in a data subdirectory from your project's top / working directory. Create the subdirectory if it doesn't exist, or modify the code accordingly.


unzip("673598238_T_ONTIME_REPORTING.zip")
mydf <- read.csv("673598238_T_ONTIME_REPORTING.csv", sep = ",", quote="\"")
mylookup <- read.csv("L_UNIQUE_CARRIERS.csv_", quote="\"", sep = "," )

head(mydf)
head(mylookup)

# All flight delay information with Airline names added
joined_df <- merge(mydf, mylookup, by.x = "OP_UNIQUE_CARRIER", by.y = "Code", all.x = TRUE, all.y = FALSE)
head(joined_df)

#### dplyr ####
# Sample dplyr code to merge 2 data frames on a single common column
# Include all rows from one data frame and only matching rows from the other

library(dplyr)
library(readr)
mytibble <- read_csv("673598238_T_ONTIME_REPORTING.zip")
mylookup_tibble <- read_csv("L_UNIQUE_CARRIERS.csv_")

joined_tibble <- left_join(mytibble, mylookup_tibble, 
                           by = c("OP_UNIQUE_CARRIER" = "Code"))
glimpse(joined_tibble)


#### data.table #### 
### Sample data.table code for the same joining of those two files:

library(data.table)
mydt <- fread('unzip -cq 673598238_T_ONTIME_REPORTING.zip')
mylookup_dt <- fread("L_UNIQUE_CARRIERS.csv_")

# merge() works with data.table objects also, but much faster
joined_dt1 <- merge(mydt, mylookup_dt, by.x = "OP_UNIQUE_CARRIER", by.y = "Code", all.x = TRUE, all.y = FALSE)

# Another way of joining data.table objects using setkey()
setkey(mydt, "OP_UNIQUE_CARRIER")
setkey(mylookup_dt, "Code")
joined_dt2 <- mylookup_dt[mydt]

# or without separate setkey() function:
joined_dt3 <- mylookup_dt[mydt, on = c(Code = "OP_UNIQUE_CARRIER")]

#### dplyr syntax + data.table speed with the dtplyr package

# If needed: 
# install.packages("dtplyr")
library(dtplyr)
my_lazy_dt <- readr::read_csv("673598238_T_ONTIME_REPORTING.zip") %>%
  dtplyr::lazy_dt()  
my_lazy_lookup <- readr::read_csv("L_UNIQUE_CARRIERS.csv_") %>%
  dtplyr::lazy_dt()  

# Check class of object
class(my_lazy_lookup)

# Join the usual dplyr ways
my_joined_lazy_dt <- left_join(my_lazy_dt, my_lazy_lookup, by = c("OP_UNIQUE_CARRIER" = "Code"))

# See class of resulting joined object
class(my_joined_lazy_dt)

# Print the object
print(my_joined_lazy_dt)

# Turn it into a tibble
myresults <- as_tibble(my_joined_lazy_dt)

# Piped code so you don't have an extra copy of the data:

joined_tibble <- left_join(my_lazy_dt, my_lazy_lookup, 
                           by = c("OP_UNIQUE_CARRIER" = "Code")) %>%
  as_tibble()

#### Other types of merges ####

# This section uses two new data sets with data by U.S. Zip code.

# Download data from https://www.zillow.com/research/data/ and https://simplemaps.com/data/us-zips
# See InfoWorld article at https://www.infoworld.com/article/3454356/how-to-merge-data-in-r-3-ways-base-r-dplyr-and-datatable.html for more details.

# Import data
# With fread I set Zip code column class to be character so five-digit codes starting with zero don't end up as four-digit numbers.
home_values_dt <- fread("Zip_MedianValuePerSqft_AllHomes.csv", 
                        colClasses = c(RegionName = "character"))
pop_density_dt <- fread("simplemaps_uszips_basicv1.6/uszips.csv", 
                        colClasses = c(zip = "character"))


#### Merges if you only want rows with matches: ####

# Base R and data.table
matches1 <- merge(home_values_dt, pop_density_dt, by.x = "RegionName", by.y = "zip", all = FALSE)

# With data.table bracket syntax
setkey(home_values_dt, RegionName)
setkey(pop_density_dt, zip)
only_matches_dt <- home_values_dt[pop_density_dt, nomatch = 0]


# dplyr
# Import data
home_values <- read_csv("Zip_MedianValuePerSqft_AllHomes.csv")
pop_density <- read_csv("simplemaps_uszips_basicv1.6/uszips.csv")

# Merge data returning only matching rows with inner join
matches2 <- inner_join(home_values, pop_density, c("RegionName" = "zip"))

#### Merge returning ALL rows from both data sets, whether matches or not

# Base R and data.table
all_rows1 <- merge(home_values_dt, pop_density_dt, by.x = "RegionName", by.y = "zip", all = TRUE)

# dplyr full join
all_rows2 <- full_join(home_values, pop_density, by = c("RegionName" = "zip"))

#### Merge returning rows in one data set that don't match the other

# I suggest dplyr's anti_join()
# All rows in pop_density that don't have a match in home_values
pop_density_no_match <- anti_join(pop_density, home_values, by = c("zip" = "RegionName"))

# All rows in home_values that don't have a match in pop_density
home_values_no_match <- anti_join(home_values, pop_density, by = c("RegionName" = "zip"))


# data.table

# All rows in home_values_dt that don't have a match in pop_density_dt:
home_values_no_match <- home_values_dt[!pop_density_dt]

# All rows in pop_density_dt that don't have a match in home_values_dt:
pop_density_no_match <- pop_density_dt[!home_values_dt]

#### Merge on multiple columns ####

# Base R and data.table merge() by two columns

merge2_dt <- merge(home_values_dt, pop_density_dt, by.x = c("RegionName", "City"), by.y = c("zip", "city"), all.x = TRUE, all.y = FALSE)

# dplyr merge by two columns
merge2_tibble <- left_join(home_values, pop_density, c("RegionName" = "zip", "City" = "city"))

# data.table syntax to merge by two columns

setkey(home_values_dt, RegionName, City)
# or 
setkeyv(home_values_dt, c("RegionName", "City"))

setkey(pop_density_dt, zip, city)
# or 
setkeyv(pop_density_dt, c("zip", "city"))

# Then use bracket syntax as usual such as
myresults <- pop_density_dt[home_values_dt]

# or 
merge2_dt <- pop_density_dt[home_values_dt, on = c(zip = "RegionName", city = "City") ]


